--[[
  Various functions for handling strings, tables and printing info to chat window.
--]]

DebugFrame = DEFAULT_CHAT_FRAME
datear = date("*t")
isdst = datear.isdst

function SetDebugFrame(nr)
  DebugFrame = getglobal("ChatFrame"..nr)
end

function echo(str)
  DEFAULT_CHAT_FRAME:AddMessage(str)
end

function debug(str)
  local oldecho = echo
  echo = function(str) DebugFrame:AddMessage(str, NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b) end
  print(str)
  echo = oldecho
end

function print(table, rowname, level) -- like Sea.io.printTable, but output is neater ;)
  if level == nil then
    if type(table) == "table" then level = 1
    else level = 0 end
  end

  local msg = string.rep("  ", level)
  if not rowname then rowname = ""
  else rowname = rowname.." = " end
  
  if table == nil then echo(msg..rowname.."nil") return end
  if type(table) == "table" then
    echo(msg..rowname.."{")
    for k, v in table do
      print(v, k, level + 1)
    end
    echo(msg.."}")
  elseif type(table) == "function" then
    echo(msg..rowname.."{{Function}}")
  elseif type(table) == "userdata" then
    echo(msg..rowname.."{{UserData}}")
  elseif type(table) == "boolean" then
    local value = "true"
    if not table then value = "false" end
    echo(msg..rowname..value)
  else
    echo(msg..rowname..table)
  end
end

SlashCmdList["PRINT"] = function(msg) RunScript("printmsg = ("..msg..")") print(printmsg) end
SLASH_PRINT1 = "/print"
SLASH_PRINT2 = "/z"


ScriptErrors.Show = function()
  echo(ScriptErrors_Message:GetText())
end


function nil2false(val)
  if val == nil then return false else return val end
end

function nil2zero(val)
  if val == nil then return 0 else return val end
end

function nil2string(val)
  if val == nil then return "" else return val end
end

function string2nil(val)
  if val == "" then return nil else return val end
end
function zero2nil(val)
  if val == 0 then return nil else return val end
end


string.split = function (str, delim)
  local result, i, j = { }, 0, 0
  
  repeat
    i, j = string.find(str, delim, 1, true)
    if i == nil then
      i = string.len(str) + 1
      j = string.len(str)
    end
    table.insert(result, string.sub(str, 1, i - 1))
    str = string.sub(str, j + 1)
  until str == ""

  return result
end

string.join = table.concat

string.ltrim = function (str)
  return string.sub(str, string.find(str, "%S", 1), string.len(str))
end

string.splitat = function (str, pos)
  return string.sub(str, 1, pos), string.sub(str, pos, string.len(str))
end





table.append = function (tdest, tsrc)
  for _, value in tsrc do
    table.insert(tdest, value)
  end
end

table.copy = function (tsrc)
  local tdest = { }
  
  for key, value in tsrc do
    if (type(value) == "table") then
      tdest[key] = table.copy(value)
    else
      tdest[key] = value
    end
  end
  
  return tdest
end

function table.merge(t1, t2)
  local tdest = { }
  for key, value in t1 do tdest[key] = value end
  for key, value in t2 do tdest[key] = value end
  return tdest
end

function table.getkey(t, func)
  local funcval = func
  if type(func) ~= "function" then func = function(v) return v == funcval end end
  if t then
    for key, value in t do
      if func(value) then return key end
    end
  end
end

function table.keys(t)
  local keys = { }
  for key, _ in t do
    table.insert(keys, key)
  end
  return keys
end




-- Takes a timestamp and an hour and min and creates new timestamp.
-- Changes date to the day after if new timestamp is more than 6 hours before timestamp
function offsetTimestamp(timestamp, hour, min)
  if not timestamp or not hour or not min then return nil end
  local datear = date("*t", tonumber(timestamp))
  local newtimestamp = time({ day = datear.day, month = datear.month, year = datear.year, hour = hour, min = min, isdst = isdst })
  if newtimestamp + 3600 * 6 < timestamp then newtimestamp = time({ day = datear.day + 1, month = datear.month, year = datear.year, hour = hour, min = min, isdst = isdst }) end
  return newtimestamp
end


